@@ -23,11 +23,11 @@ from typing import (
 import click
 from jsonschema import Draft4Validator, SchemaError, ValidationError
 
+from singer_sdk.configuration._dict_config import parse_environment_config
 from singer_sdk.exceptions import ConfigValidationError
 from singer_sdk.helpers._classproperty import classproperty
 from singer_sdk.helpers._compat import metadata
 from singer_sdk.helpers._secrets import SecretString, is_common_secret_key
-from singer_sdk.helpers._typing import is_string_array_type
 from singer_sdk.helpers._util import read_json_file
 from singer_sdk.helpers.capabilities import (
     FLATTENING_CONFIG,
@@ -61,7 +61,19 @@ class PluginBase(metaclass=abc.ABCMeta):
         Returns:
             Plugin logger.
         """
-        return logging.getLogger(cls.name)
+        # Get the level from <PLUGIN_NAME>_LOGLEVEL or LOGLEVEL environment variables
+        LOGLEVEL = (
+            os.environ.get(f"{cls.name.upper()}_LOGLEVEL")
+            or os.environ.get("LOGLEVEL")
+            or "INFO"
+        ).upper()
+
+        assert (
+            LOGLEVEL in logging._levelToName.values()
+        ), f"Invalid LOGLEVEL configuration: {LOGLEVEL}"
+        logger = logging.getLogger(cls.name)
+        logger.setLevel(LOGLEVEL)
+        return logger
 
     # Constructor
 
@@ -132,32 +144,12 @@ class PluginBase(metaclass=abc.ABCMeta):
 
         Returns:
             Dictionary of configuration parsed from the environment.
-
-        Raises:
-            ValueError: If there's an environment variable with unsupported syntax.
         """
-        result: Dict[str, Any] = {}
         plugin_env_prefix = f"{cls.name.upper().replace('-', '_')}_"
         config_jsonschema = cls.config_jsonschema
         cls.append_builtin_config(config_jsonschema)
-        for config_key in config_jsonschema["properties"].keys():
-            env_var_name = plugin_env_prefix + config_key.upper().replace("-", "_")
-            if env_var_name in os.environ:
-                env_var_value = os.environ[env_var_name]
-                cls.logger.info(
-                    f"Parsing '{config_key}' config from env variable '{env_var_name}'."
-                )
-                if is_string_array_type(config_jsonschema["properties"][config_key]):
-                    if env_var_value[0] == "[" and env_var_value[-1] == "]":
-                        raise ValueError(
-                            "A bracketed list was detected in the environment variable "
-                            f"'{env_var_name}'. This syntax is no longer supported. "
-                            "Please remove the brackets and try again."
-                        )
-                    result[config_key] = env_var_value.split(",")
-                else:
-                    result[config_key] = env_var_value
-        return result
+
+        return parse_environment_config(config_jsonschema, plugin_env_prefix)
 
     # Core plugin metadata:
 
@@ -283,23 +275,24 @@ class PluginBase(metaclass=abc.ABCMeta):
             print_fn: A function to use to display the plugin version.
                 Defaults to :function:`print`.
         """
-        print_fn(f"{cls.name} v{cls.plugin_version}, Meltano SDK v{cls.sdk_version})")
+        print_fn(f"{cls.name} v{cls.plugin_version}, Meltano SDK v{cls.sdk_version}")
 
-    def _get_about_info(self) -> Dict[str, Any]:
+    @classmethod
+    def _get_about_info(cls: Type["PluginBase"]) -> Dict[str, Any]:
         """Returns capabilities and other tap metadata.
 
         Returns:
             A dictionary containing the relevant 'about' information.
         """
         info: Dict[str, Any] = OrderedDict({})
-        info["name"] = self.name
-        info["description"] = self.__doc__
-        info["version"] = self.plugin_version
-        info["sdk_version"] = self.sdk_version
-        info["capabilities"] = self.capabilities
+        info["name"] = cls.name
+        info["description"] = cls.__doc__
+        info["version"] = cls.plugin_version
+        info["sdk_version"] = cls.sdk_version
+        info["capabilities"] = cls.capabilities
 
-        config_jsonschema = self.config_jsonschema
-        self.append_builtin_config(config_jsonschema)
+        config_jsonschema = cls.config_jsonschema
+        cls.append_builtin_config(config_jsonschema)
         info["settings"] = config_jsonschema
         return info
 
@@ -333,13 +326,14 @@ class PluginBase(metaclass=abc.ABCMeta):
         if PluginCapabilities.FLATTENING in capabilities:
             _merge_missing(FLATTENING_CONFIG, config_jsonschema)
 
-    def print_about(self, format: Optional[str] = None) -> None:
+    @classmethod
+    def print_about(cls: Type["PluginBase"], format: Optional[str] = None) -> None:
         """Print capabilities and other tap metadata.
 
         Args:
             format: Render option for the plugin information.
         """
-        info = self._get_about_info()
+        info = cls._get_about_info()
 
         if format == "json":
             print(json.dumps(info, indent=2, default=str))
