@@ -27,13 +27,13 @@ from typing import (
 import pendulum
 import requests
 import singer
-from singer import RecordMessage, SchemaMessage, StateMessage
-from singer.schema import Schema
+from singer import RecordMessage, Schema, SchemaMessage, StateMessage
 
 from singer_sdk.exceptions import InvalidStreamSortException, MaxRecordsLimitException
 from singer_sdk.helpers._catalog import pop_deselected_record_properties
 from singer_sdk.helpers._compat import final
 from singer_sdk.helpers._flattening import get_flattening_options
+from singer_sdk.helpers._schema import SchemaPlus
 from singer_sdk.helpers._singer import (
     Catalog,
     CatalogEntry,
@@ -304,6 +304,29 @@ class Stream(metaclass=abc.ABCMeta):
         state = self.get_context_state(context)
         write_replication_key_signpost(state, value)
 
+    def compare_start_date(self, value: str, start_date_value: str) -> str:
+        """Compare a bookmark value to a start date and return the most recent value.
+
+        If the replication key is a datetime-formatted string, this method will parse
+        the value and compare it to the start date. Otherwise, the bookmark value is
+        returned.
+
+        If the tap uses a non-datetime replication key (e.g. an UNIX timestamp), the
+        developer is encouraged to override this method to provide custom logic for
+        comparing the bookmark value to the start date.
+
+        Args:
+            value: The replication key value.
+            start_date_value: The start date value from the config.
+
+        Returns:
+            The most recent value between the bookmark and start date.
+        """
+        if self.is_timestamp_replication_key:
+            return max(value, start_date_value, key=pendulum.parse)
+        else:
+            return value
+
     def _write_starting_replication_value(self, context: Optional[dict]) -> None:
         """Write the starting replication value, if available.
 
@@ -320,8 +343,13 @@ class Stream(metaclass=abc.ABCMeta):
             ):
                 value = replication_key_value
 
-            elif "start_date" in self.config:
-                value = self.config["start_date"]
+            # Use start_date if it is more recent than the replication_key state
+            start_date_value: Optional[str] = self.config.get("start_date")
+            if start_date_value:
+                if not value:
+                    value = start_date_value
+                else:
+                    value = self.compare_start_date(value, start_date_value)
 
         write_starting_replication_value(state, value)
 
@@ -502,7 +530,7 @@ class Stream(metaclass=abc.ABCMeta):
         return CatalogEntry(
             tap_stream_id=self.tap_stream_id,
             stream=self.name,
-            schema=Schema.from_dict(self.schema),
+            schema=SchemaPlus.from_dict(self.schema),
             metadata=self.metadata,
             key_properties=self.primary_keys or [],
             replication_key=self.replication_key,
@@ -817,7 +845,7 @@ class Stream(metaclass=abc.ABCMeta):
 
         if extra_tags:
             metric["tags"].update(extra_tags)
-        self._metric_logging_function(f"INFO METRIC: {str(metric)}")
+        self._metric_logging_function(f"INFO METRIC: {json.dumps(metric)}")
 
     def _write_record_count_log(
         self, record_count: int, context: Optional[dict]
