@@ -249,7 +249,7 @@ class Target(PluginBase, SingerReader, metaclass=abc.ABCMeta):
 
     # Message handling
 
-    def _process_lines(self, file_input: IO[str]) -> Counter[SingerMessageType]:
+    def _process_lines(self, file_input: IO[str]) -> Counter[str]:
         """Internal method to process jsonl lines from a Singer tap.
 
         Args:
@@ -273,7 +273,7 @@ class Target(PluginBase, SingerReader, metaclass=abc.ABCMeta):
 
     def _process_endofpipe(self) -> None:
         """Called after all input lines have been read."""
-        self.drain_all()
+        self.drain_all(is_endofpipe=True)
 
     def _process_record_message(self, message_dict: dict) -> None:
         """Process a RECORD message.
@@ -403,15 +403,27 @@ class Target(PluginBase, SingerReader, metaclass=abc.ABCMeta):
     # Sink drain methods
 
     @final
-    def drain_all(self) -> None:
+    def drain_all(self, is_endofpipe: bool = False) -> None:
         """Drains all sinks, starting with those cleared due to changed schema.
 
         This method is internal to the SDK and should not need to be overridden.
+
+        Args:
+            is_endofpipe: This is passed by the
+                          :meth:`~singer_sdk.Sink._process_endofpipe()` which
+                          is called after the target instance has finished
+                          listening to the stdin
         """
         state = copy.deepcopy(self._latest_state)
         self._drain_all(self._sinks_to_clear, 1)
+        if is_endofpipe:
+            for sink in self._sinks_to_clear:
+                sink.clean_up()
         self._sinks_to_clear = []
         self._drain_all(list(self._sinks_active.values()), self.max_parallelism)
+        if is_endofpipe:
+            for sink in self._sinks_active.values():
+                sink.clean_up()
         self._write_state_message(state)
         self._reset_max_record_age()
 
@@ -500,10 +512,11 @@ class Target(PluginBase, SingerReader, metaclass=abc.ABCMeta):
 
             if not about:
                 cls.print_version(print_fn=cls.logger.info)
+            else:
+                cls.print_about(format=format)
+                return
 
             validate_config: bool = True
-            if about:
-                validate_config = False
 
             cls.print_version(print_fn=cls.logger.info)
 
@@ -530,10 +543,7 @@ class Target(PluginBase, SingerReader, metaclass=abc.ABCMeta):
                 validate_config=validate_config,
             )
 
-            if about:
-                target.print_about(format)
-            else:
-                target.listen(file_input)
+            target.listen(file_input)
 
         return cli
 
